
###############
# 解説1
###############
import ctypes 
import tisgrabber as tis
import cv2
import numpy as np

ic = ctypes.cdll.LoadLibrary("./tisgrabber_x64.dll")
tis.declareFunctions(ic)
ic.IC_InitLibrary(0)


###############
# 解説2
###############
#新しいグラバーハンドルを作成します。
hGrabber = ic.IC_CreateGrabber()
#デバイスを選択
ic.IC_OpenDevByUniqueName(hGrabber, tis.T("DFK 33UX226 10420839"))
ic.IC_SetVideoFormat(hGrabber, tis.T("Y800 (2000x1500)"))
ic.IC_SetFrameRate(hGrabber, ctypes.c_float( 10.0))

###############
# 解説3
###############
if(ic.IC_IsDevValid(hGrabber)):
    # メモリ内で受け取るピクセルフォーマットを設定します。
    ic.IC_SetFormat(hGrabber, tis.SinkFormats.Y800)
    ic.IC_StartLive(hGrabber, 0)
    key = ""
    while key != "q":
        print("p: 画像保存")
        print("q: プログラム終了")
        key = input('pかqか入力してください。:')
        if key == "p":
            #2秒間カメラから画像を送られてくるのをを待つ
            if ic.IC_SnapImage(hGrabber, 2000) == tis.IC_SUCCESS:
                # 画像の解像度・フォーマットを宣言
                Width = ctypes.c_long()
                Height = ctypes.c_long()
                BitsPerPixel = ctypes.c_int()
                colorformat = ctypes.c_int()

                # 画像の解像度・フォーマットを取得する
                ic.IC_GetImageDescription(hGrabber, Width, Height,
                                          BitsPerPixel, colorformat)

                # バッファサイズを計算し、ピクセルあたりのバイト数と
                # numpy配列のデータ型を取得します。
                elementsperpixel = 1
                dtype = np.uint8
                if colorformat.value == tis.SinkFormats.Y800:
                    elementsperpixel = 1  # 1 byte per pixel
                if colorformat.value == tis.SinkFormats.Y16:
                    dtype = np.uint16
                    elementsperpixel = 1  # 1 uint16 per pixel
                if colorformat.value == tis.SinkFormats.RGB24:
                    elementsperpixel = 3  # BGR format, 3 bytes
                if colorformat.value == tis.SinkFormats.RGB32:
                    elementsperpixel = 4  # BGRA format, 4 bytes

                print(colorformat.value)
                buffer_size = Width.value * Height.value * int(float(BitsPerPixel.value) / 8.0)

                # イメージデータ取得
                imagePtr = ic.IC_GetImagePtr(hGrabber)

                imagedata = ctypes.cast(imagePtr,
                                        ctypes.POINTER(ctypes.c_ubyte *
                                                       buffer_size))

                # numpy配列取得
                image = np.ndarray(buffer=imagedata.contents,
                                   dtype=dtype,
                                   shape=(Height.value,
                                          Width.value,
                                          elementsperpixel))

                # OpenCVの関数
                image = cv2.flip(image, 0)

                cv2.imwrite("image.bmp",image)
            
                print("画像を保存しました。")
                cv2.waitKey(10)
            else:
                print("2秒間フレームを受け取っていません。")

    ic.IC_StopLive(hGrabber)

 

else:
    ic.IC_MsgBox(tis.T("デバイスを開けませんでした。"), tis.T("Simple Live Video"))

ic.IC_ReleaseGrabber(hGrabber)
