import asyncio

frame_queue = asyncio.Queue()

# カウンターの初期化
capture_count = 0
process_count = 0

async def capture_frames():
    """外部トリガで撮像し、画像をキューに追加するタスク"""
    global capture_count
    while True:
        await asyncio.sleep(0.0083)
        frame = "captured_frame_data"
        await frame_queue.put(frame)
        capture_count += 1  # 画像取得カウント
        print(f"Frame captured and added to queue. Total captured: {capture_count}")

async def process_frame(frame):
    """個別のフレーム処理を行う関数"""
    global process_count
    print("Processing frame...")
    await asyncio.sleep(0.05)  # 例として50msの処理時間
    process_count += 1  # 画像処理カウント
    print(f"Frame processed. Total processed: {process_count}")

async def process_frames():
    """キューから画像を取得し、非同期で複数タスクで処理を行うタスク"""
    while True:
        frame = await frame_queue.get()
        # 画像処理タスクを並列に実行
        asyncio.create_task(process_frame(frame))
        frame_queue.task_done()

async def main():
    capture_task = asyncio.create_task(capture_frames())
    process_task = asyncio.create_task(process_frames())
    await asyncio.gather(capture_task, process_task)

asyncio.run(main())