
###############
# 解説1
###############
import ctypes 
import tisgrabber as tis
import cv2
import numpy as np

ic = ctypes.cdll.LoadLibrary("./tisgrabber_x64.dll")
tis.declareFunctions(ic)
ic.IC_InitLibrary(0)


###############
# 解説2
###############
#新しいグラバーハンドルを作成します。
hGrabber = ic.IC_CreateGrabber()
#デバイスを選択
#ic.IC_OpenDevByUniqueName(hGrabber, tis.T("DFK 33UX265 46420021"))
#ic.IC_OpenDevByUniqueName(hGrabber, tis.T("DFK 37BUR0521 37421027"))
ic.IC_OpenDevByUniqueName(hGrabber, tis.T("DMK 33GR0134 17220330"))
#ic.IC_OpenDevByUniqueName(hGrabber, tis.T("DMK 27BUJ003 42220031"))
#ic.IC_OpenDevByUniqueName(hGrabber, tis.T("DMK 32BUR0234 48420033"))
ic.IC_SetVideoFormat(hGrabber, tis.T("Y16 (640x480)"))
ic.IC_SetFrameRate(hGrabber, ctypes.c_float( 10.0))
ic.IC_SetPropertySwitch(hGrabber, tis.T("Trigger"), tis.T("Enable"), 0)
###############
# 解説3
###############
if(ic.IC_IsDevValid(hGrabber)):
    # メモリ内で受け取るピクセルフォーマットを設定します。
    ic.IC_SetFormat(hGrabber, tis.SinkFormats.Y16)
    ic.IC_StartLive(hGrabber, 0)
    key = ""
    while key != "q":
        print("p: 画像保存")
        print("q: プログラム終了")
        key = input('pかqか入力してください。:')
        if key == "p":
            #2秒間カメラから画像を送られてくるのをを待つ
            if ic.IC_SnapImage(hGrabber, 2000) == tis.IC_SUCCESS:
                # 画像の解像度・フォーマットを宣言
                Width = ctypes.c_long()
                Height = ctypes.c_long()
                BitsPerPixel = ctypes.c_int()
                colorformat = ctypes.c_int()

                # 画像の解像度・フォーマットを取得する
                ic.IC_GetImageDescription(hGrabber, Width, Height,
                                          BitsPerPixel, colorformat)

                # バッファサイズを計算し、ピクセルあたりのバイト数と
                # numpy配列のデータ型を取得します。
                elementsperpixel = 1
                dtype = np.uint8
                if colorformat.value == tis.SinkFormats.Y800:
                    elementsperpixel = 1  # 1 byte per pixel
                if colorformat.value == tis.SinkFormats.Y16:
                    dtype = np.uint16
                    elementsperpixel = 1  # 1 uint16 per pixel
                if colorformat.value == tis.SinkFormats.RGB24:
                    elementsperpixel = 3  # BGR format, 3 bytes
                if colorformat.value == tis.SinkFormats.RGB32:
                    elementsperpixel = 4  # BGRA format, 4 bytes


                buffer_size = Width.value * Height.value * int(float(BitsPerPixel.value) / 8.0)

                # イメージデータ取得
                imagePtr = ic.IC_GetImagePtr(hGrabber)

                imagedata = ctypes.cast(imagePtr,
                                        ctypes.POINTER(ctypes.c_ubyte *
                                                       buffer_size))

                # バイトデータ(最初の16バイト)
                byte_data = np.array(imagedata.contents[:16])
                print("Raw data (16byte):", byte_data)
                ##出力例
                ##Raw data (byte): [ 28 146   0 144 228 144  32 145 116 148   0 142 100 144 128 145]

                # バイトデータを2進数(8ビット)で表示
                print("Raw data (16byte, binary):",
                    [format(x, '08b') for x in byte_data])
                ##出力例
                ##Raw data (byte, binary): ['00011100', '10010010', '00000000', '10010000', '11100100', '10010000', '00100000', '10010001', '01110100', '10010100', '00000000', '10001110', '01100100', '10010000', '10000000', '10010001']
                
                # 16ビットデータとして解釈したもの(最初の8ピクセル)
                data = np.frombuffer(bytearray(imagedata.contents), dtype=np.uint16)
                print("Interpreted 16-bit data:", data[:8])
                ##出力例
                ##Interpreted 16-bit data: [37404 36864 37092 37152 38004 36352 36964 37248]

                # 16ビットデータを2進数(16ビット)で表示
                print("Interpreted 16-bit data (binary):",
                    [format(x, '016b') for x in data[:8]])
                ##出力例
                ##Interpreted 16-bit data (binary): ['1001001000011100', '1001000000000000', '1001000011100100', '1001000100100000', '1001010001110100', '1000111000000000', '1001000001100100', '1001000110000000']

                # numpy配列取得
                image = np.ndarray(buffer=imagedata.contents,
                                   dtype=dtype,
                                   shape=(Height.value,
                                          Width.value,
                                          elementsperpixel))

                # 最初のピクセルの輝度値を取得
                first_pixel_value = image[0, 0, 0]
                print(f"最初のピクセルの輝度値: {first_pixel_value}")
                ##出力例
                ##最初のピクセルの輝度値: 37404

                # OpenCVの関数
                image = cv2.flip(image, 0)

                cv2.imwrite("image.png",image)
                print("画像を保存しました。")
                cv2.waitKey(10)
            else:
                print("2秒間フレームを受け取っていません。")

    ic.IC_StopLive(hGrabber)

 

else:
    ic.IC_MsgBox(tis.T("デバイスを開けませんでした。"), tis.T("Simple Live Video"))

ic.IC_ReleaseGrabber(hGrabber)
